/*
 * Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file    edltypes.h
 * @ingroup IController
 * @brief   EDL "standard" types including known bit-length signed and unsigned integer type[def]s
 *          and definitions of EDL_TRUE and EDL_FALSE
 */

#ifndef __EDLTYPES_H__
#define __EDLTYPES_H__

#include <stddef.h> // NULL, size_t

#include "platform.h"
#include <edl/edlnamespaces.h>

_BEGIN_HQN_NAMESPACE

typedef char                int8;
typedef signed short        int16;
typedef signed int          int32;
typedef signed long long    int64;

typedef unsigned char       uint8;
typedef unsigned short      uint16;
typedef unsigned int        uint32;
typedef unsigned long long  uint64;

_END_HQN_NAMESPACE

#ifdef __cplusplus

_BEGIN_EDL_NAMESPACE

typedef _FROM_HQN_NAMESPACE(int8)   int8;
typedef _FROM_HQN_NAMESPACE(int16)  int16;
typedef _FROM_HQN_NAMESPACE(int32)  int32;
typedef _FROM_HQN_NAMESPACE(int64)  int64;

typedef _FROM_HQN_NAMESPACE(uint8)  uint8;
typedef _FROM_HQN_NAMESPACE(uint16) uint16;
typedef _FROM_HQN_NAMESPACE(uint32) uint32;
typedef _FROM_HQN_NAMESPACE(uint64) uint64;

_END_EDL_NAMESPACE

#endif // ifdef __cplusplus

#define E_FALSE 0
#define E_TRUE  1

#ifndef UNREFERENCED_PARAMETER
#ifdef _WIN32
#define UNREFERENCED_PARAMETER(P)          (P)
#else
#define UNREFERENCED_PARAMETER(P)          (void)(P)
#endif
#endif

#ifdef _WIN32
#ifdef EDL_DLL
#ifdef EDL_DLL_EXPORTS
#define EDL_API __declspec(dllexport)
#else /* EDL_DLL_EXPORTS */
#define EDL_API __declspec(dllimport)
#endif
#else /* EDL_DLL */
#define EDL_API 
#endif
#else /* _WIN32 */
#define EDL_API 
#endif

#endif /* __EDLTYPES_H__ */
