/*
* Copyright (C) 2008-2019 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef __IABORT_H__
#define __IABORT_H__

#include <edl/smartptr.h>
#include <edl/ircobject.h>
 
_BEGIN_EDL_NAMESPACE

class IAbort;
DECL_SMART_PTR (IAbort);

/**
 * @class IAbort
 * @brief A simple class to signal an abort in processing
 */
class IAbort : public IRCObject
{
    public:
        virtual ~IAbort() {}

        /**
         * @brief   Create an abort signal.
         */
        static IAbortPtr create();

        /**
         * @brief   Reset the abort object to an unsignalled state. 
         */
        virtual void resetSignal () = 0;

        /**
         * @brief   Signal that the output operation should be aborted.
         */
        virtual void signalAbort() = 0;

        /**
         * @brief   Determine if the abort has been signalled.
         */
        virtual bool abortSignalled() = 0;
};

     
// A useful macro for throwing an exception of EDL_ERR_ABORTED if the given IAbort
// notes that it has been signalled.
#define CHECKABORT(abort) THROW_IF(abort && abort->abortSignalled(), EDL_ERR_ABORTED);

    
_END_EDL_NAMESPACE

#endif /* __IABORT_H__ */


