﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="IMakoDocument.cs" company="Global Graphics Software Ltd">
//   Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
// </copyright>
// --------------------------------------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;

namespace Mako.Wrapper
{
    public interface IMakoDocument : IDisposable
    {
        void UpdateRaster(IMakoRenderer renderer, int pageIndex, MakoSize destinationSize, MakoRect rect, string[] separations, int nSeparation, ref MakoRaster raster);

        IMakoElement GetElement(int pageIndex, double x, double y, IMakoElement selectedElement);

        IEnumerable<IMakoElement> GetAllElements(int pageIndex);

        MakoSize GetPageSize(int pageIndex);

        int GetPageCount();

        IEnumerable<IMakoLayer> GetLayers();

        IEnumerable<MakoSeparation> GetSeparations();

        bool CanUndo();

        bool CanRedo();

        IMakoStateChange Undo();

        IMakoStateChange Redo();

        void StartChangeGroup();

        void CompleteChangeGroup();

        void LoadSeparations();

        bool AddSeparation(MakoSeparation makoSeparation);

        bool ReplaceSeparation(int index, MakoSeparation makoSeparation);

        void AddLayerWithName(string layerName);

        void AddElement(IMakoElement makoElement);

        IMakoElement CreatePresetShapeContour(int pageIndex, MakoShapeType type, MakoPoint[] points, MakoContourSettings settings, MakoSeparation separation);

        IMakoElement CreateCutContour(IntPtr[] elementHandles, MakoContourSettings settings, MakoSeparation separation);
    }
}