﻿// -----------------------------------------------------------------------
//  <copyright file="MakoDll.cs" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------


using System;
using System.Runtime.InteropServices;

namespace Mako.Wrapper
{
    public enum MakoShapeType
    {
        Ellipse,
        Polygon
    }

    public enum MakoContourAlignment
    {
        Outline,
        BoundingBox
    }

    public static class MakoDll
    {
        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool fnInitialize([MarshalAs(UnmanagedType.LPWStr)] string libraryLocation);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnShutdown();

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int fnOpenAssembly([MarshalAs(UnmanagedType.LPWStr)] string assemblyFile, ref IntPtr assemblyHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnCloseAssembly(IntPtr assemblyHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern void fnSaveAssembly(IntPtr assemblyHandle, string filePath, string[] outputParams, int parameterCount);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern void fnSaveAssemblyAsPdf(IntPtr assemblyHandle, string filePath, bool incremental);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr fnOpenDocument(IntPtr assemblyHandle, int documentIndex);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnCloseDocument(IntPtr documentHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool fnCanUndo(IntPtr documentHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool fnCanRedo(IntPtr documentHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool fnUndo(IntPtr documentHandle, 
            [MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] out IntPtr[] changedElements, out int changedElementsLength, 
            [MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 4)] out int[] changedPageIndexes, out int changedPageIndexesLength, 
            ref MakoStateChangeType changeType);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool fnRedo(IntPtr documentHandle,
            [MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] out IntPtr[] changedElements, out int changedElementsLength,
            [MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 4)] out int[] changedPageIndexes, out int changedPageIndexesLength,
            ref MakoStateChangeType changeType);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr fnCreateRenderer();

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr fnCloseRenderer(IntPtr rendererHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode, SetLastError = false)]
        public static extern void fnGetRaster(IntPtr rendererHandle, IntPtr documentHandle, int pageIndex, MakoSize destinationSize, MakoRect rect, string[] separations, int nSeparation, ref MakoRaster raster);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnFreeRaster(MakoRaster raster);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int fnGetPageCount(IntPtr documentHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern double fnGetPageWidth(IntPtr documentHandle, int pageIndex);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern double fnGetPageHeight(IntPtr documentHandle, int pageIndex);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr fnGetElement(IntPtr documentHandle, int pageIndex, double x, double y, IntPtr elementHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnGetAllElements(IntPtr documentHandle, int pageIndex, [MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 3)] out IntPtr[] elements, out int elementCount);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr fnCreateContour(IntPtr documentHandle, IntPtr[] elementHandles, int length, MakoContourSettings settings, MakoSeparation separation);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr fnCreatePresetShapeContour(IntPtr documentHandle, int pageIndex, MakoShapeType type, MakoPoint[] points, int length, MakoContourSettings settings, MakoSeparation separation);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnAddElement(IntPtr documentHandle, IntPtr elementHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr fnGetElementNode(IntPtr elementHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int fnGetElementPageIndex(IntPtr elementHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnGetElementBoundingBox(IntPtr elementHandle, ref MakoRect rect);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnSetElementBoundingBox(IntPtr elementHandle, MakoRect rect, bool isInvertedHorizontally, bool isInvertedVertically);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnExtractElements([MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 1)] IntPtr[] elementHandle, int count);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr fnCopyElement(IntPtr elementHandle, int newPageIndex);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnCloseElement(IntPtr elementHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.LPWStr)]
        public static extern string fnGetLayerName(IntPtr layerHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int fnGetLayerCount(IntPtr documentHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr fnGetLayer(IntPtr documentHandle, int layerIndex);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnFreeLayer(IntPtr layerHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool fnGetLayerIsVisible(IntPtr layerHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnSetLayerIsVisible(IntPtr layerHandle, bool isVisible);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool fnGetLayerIsLocked(IntPtr layerHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnSetLayerIsLocked(IntPtr layerHandle, bool isLocked);
 
        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool fnGetLayerIsPrintVisible(IntPtr layerHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int fnLoadSeparations(IntPtr documentHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int fnGetSeparationCount(IntPtr documentHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool fnGetSeparation(IntPtr documentHandle, int index, ref MakoSeparation separation);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool fnAddSeparation(IntPtr documentHandle, MakoSeparation separation);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool fnReplaceSeparation(IntPtr documentHandle, int index, MakoSeparation separation);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnAddLayerWithName(IntPtr documentHandle, [MarshalAs(UnmanagedType.LPWStr)] string layerName);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnStartChangeGroup(IntPtr documentHandle);

        [DllImport("MakoWrapperDll.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnCompleteChangeGroup(IntPtr documentHandle);
    }

    public class MakoDllException : Exception
    {
        public int ErrorCode { get; }

        public MakoDllException(string message, int errorCode)
            : base($"An unexpected error occurred in Mako: {message}, error: {errorCode}")
        {
            ErrorCode = errorCode;
        }
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct MakoContourSettings
    {
        public double offsetInPoints;
        [MarshalAs(UnmanagedType.I1)]
        public bool removeHoles;
        [MarshalAs(UnmanagedType.LPWStr)]
        public string layerName;
        public MakoContourAlignment alignment;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct MakoRaster
    {
        public IntPtr buffer;
        public int bufferStride;
        public int width;
        public int height;
        public int samplePerPixel;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct MakoMatrix
    {
        public double m11;
        public double m12;
        public double m21;
        public double m22;
        public double offsetX;
        public double offsetY;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct MakoPoint
    {
        public double x;
        public double y;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct MakoRect
    {
        public double x;
        public double y;
        public double width;
        public double height;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct MakoSize
    {
        public double width;
        public double height;
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
    public struct MakoSeparation
    {
        public string name;
        [MarshalAs(UnmanagedType.I1)]
        public bool isProcessColor;
        public int colorArgbValue;
    }
}
