﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="MakoInstance.cs" company="Global Graphics Software Ltd">
//   Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
// </copyright>
// --------------------------------------------------------------------------------------------------------------------

// ReSharper disable InconsistentNaming
// ReSharper disable MemberCanBeMadeStatic.Global

using System;
using System.IO;
using System.Reflection;

namespace Mako.Wrapper
{
    /// <summary>
    /// A C# class allowing access to MakoInstance functionality.
    /// </summary>
    public class MakoInstance
    {
        private static readonly object m_creationLock = new object();
        
        private static MakoInstance m_singleton;
        private bool m_isInitialized;

        public static MakoInstance Instance
        {
            get
            {
                lock (m_creationLock)
                    return m_singleton ?? (m_singleton = new MakoInstance());
            }
        }

        private MakoInstance()
        {
            Initialize();
        }

        private void Initialize()
        {
            try
            {
                var coreSupportLibraryLocation = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
                m_isInitialized = MakoDll.fnInitialize(coreSupportLibraryLocation);

                if (m_isInitialized)
                    return;

                var up = new Exception("Call to fnInitialize() returned an error.");
                Console.WriteLine(up);
                throw up;
            }
            catch (DllNotFoundException ex)
            {
                Console.WriteLine(ex);
                throw;
            }
        }

        ~MakoInstance()
        {
            Shutdown();
        }

        public IMakoAssembly Open(string assemblyFile)
        {
            var assemblyHandle = IntPtr.Zero;
            var result = MakoDll.fnOpenAssembly(assemblyFile, ref assemblyHandle);
            if (result != 0 || assemblyHandle == IntPtr.Zero)
                throw new MakoDllException("Failed to open assembly.", result);

            return new MakoAssembly(assemblyHandle);
        }

        public MakoRenderer CreateRenderer()
        {
            return new MakoRenderer(MakoDll.fnCreateRenderer());
        }

        public void Shutdown()
        {
            if (!m_isInitialized)
                return;

            m_isInitialized = false;
            MakoDll.fnShutdown();
        }
    }
}