﻿// -----------------------------------------------------------------------
//  <copyright file="MakoStateChange.cs" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;

namespace Mako.Wrapper
{
    public class MakoStateChange : IMakoStateChange
    {
        public MakoStateChange(MakoStateChangeType changeType, IEnumerable<IntPtr> changedElements, IEnumerable<int> changedPageIndexes)
        {
            if (changedElements == null)
                throw new ArgumentNullException(nameof(changedElements));

            ChangeType = changeType;
            ChangedElements = changedElements.Select(e => new MakoElement(e));
            ChangedPageIndexes = changedPageIndexes ?? throw new ArgumentNullException(nameof(changedPageIndexes));
        }

        public static IMakoStateChange None => new MakoStateChange(MakoStateChangeType.None, new IntPtr[] { }, new int[] { });

        public MakoStateChangeType ChangeType { get; }

        public IEnumerable<IMakoElement> ChangedElements { get; }

        public IEnumerable<int> ChangedPageIndexes { get; }
    }

    [Flags]
    public enum MakoStateChangeType
    {
        None = 0,
        Pages = 1,
        Separations = 2,
        Layers = 4
    }
}