// -----------------------------------------------------------------------
//  <copyright file="MakoWrapperDll.h" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#ifndef __MAKOWRAPPERDLL_H__
#define __MAKOWRAPPERDLL_H__

#ifdef MAKOWRAPPER_EXPORTS
#define MAKOWRAPPER_API __declspec(dllexport)
#else
#define MAKOWRAPPER_API __declspec(dllimport)
#endif

extern "C"
{
    // Mako APIs
    MAKOWRAPPER_API bool fnInitialize(wchar_t* libraryLocation);
    MAKOWRAPPER_API void fnShutdown();

    // Assembly APIs
    MAKOWRAPPER_API MakoResult fnOpenAssembly(wchar_t* filePath, MakoHandle* assemblyHandle);
    MAKOWRAPPER_API void fnCloseAssembly(MakoHandle assemblyHandle);
    MAKOWRAPPER_API void fnSaveAssembly(MakoHandle assemblyHandle, wchar_t* filePath, wchar_t* outputParams[], int parameterCount);
    MAKOWRAPPER_API void fnSaveAssemblyAsPdf(MakoHandle assemblyHandle, wchar_t* filePath, bool incremental);

    // Document APIs
    MAKOWRAPPER_API MakoHandle fnOpenDocument(MakoHandle assemblyHandle, int documentIndex);
    MAKOWRAPPER_API void fnCloseDocument(MakoHandle documentHandle);

    // Page APIs
    MAKOWRAPPER_API double fnGetPageWidth(MakoHandle documentHandle, int pageIndex);
    MAKOWRAPPER_API double fnGetPageHeight(MakoHandle documentHandle, int pageIndex);
    MAKOWRAPPER_API int fnGetPageCount(MakoHandle documentHandle);

    // Rendering APIs
    MAKOWRAPPER_API MakoHandle fnCreateRenderer();
    MAKOWRAPPER_API void fnCloseRenderer(MakoHandle rendererHandle);
    MAKOWRAPPER_API void fnGetRaster(MakoHandle rendererHandle, MakoHandle documentHandle, int pageIndex, MakoSize destinationSize, MakoRect sourceRect, wchar_t** separations, int nSeparation, MakoRaster* raster);
    MAKOWRAPPER_API void fnFreeRaster(MakoRaster raster);

    // Undo Redo APIs
    MAKOWRAPPER_API bool fnCanUndo(MakoHandle documentHandle);
    MAKOWRAPPER_API bool fnCanRedo(MakoHandle documentHandle);
    MAKOWRAPPER_API bool fnUndo(MakoHandle documentHandle, MakoHandle** changedElements, int* changedElementsSize, int** changedPageIndexes, int* changedPageIndexesSize, eStateChangeType* changeType);
    MAKOWRAPPER_API bool fnRedo(MakoHandle documentHandle, MakoHandle** changedElements, int* changedElementsSize, int** changedPageIndexes, int* changedPageIndexesSize, eStateChangeType* changeType);
    MAKOWRAPPER_API void fnStartChangeGroup(MakoHandle documentHandle);
    MAKOWRAPPER_API void fnCompleteChangeGroup(MakoHandle documentHandle);

    // Contour APIs
    MAKOWRAPPER_API MakoHandle fnCreateContour(MakoHandle documentHandle, MakoHandle elementHandles[], int count, const MakoContourSettings& settings, const MakoSeparation& separation);
    MAKOWRAPPER_API MakoHandle fnCreatePresetShapeContour(MakoHandle documentHandle, int pageIndex, eShapeType type, MakoPoint points[], int length, const MakoContourSettings& settings, const MakoSeparation& separation);

    // Element APIs
    MAKOWRAPPER_API MakoHandle fnGetElement(MakoHandle documentHandle, int pageIndex, double x, double y, MakoHandle selectedElementHandle);
    MAKOWRAPPER_API void fnAddElement(MakoHandle documentHandle, MakoHandle elementHandle);
    MAKOWRAPPER_API MakoHandle fnGetElementNode(MakoHandle elementHandle);
    MAKOWRAPPER_API void fnExtractElements(MakoHandle* elementHandles, int count);
    MAKOWRAPPER_API int fnGetElementPageIndex(MakoHandle elementHandle);
    MAKOWRAPPER_API void fnGetElementBoundingBox(MakoHandle elementHandle, MakoRect* rect);
    MAKOWRAPPER_API void fnSetElementBoundingBox(MakoHandle elementHandle, MakoRect rect, bool isInvertedHorizontally, bool isInvertedVertically);
    MAKOWRAPPER_API void fnGetAllElements(MakoHandle documentHandle, int pageIndex, MakoHandle** elements, int* elementCount);
    MAKOWRAPPER_API MakoHandle fnCopyElement(MakoHandle elementHandle, int newPageIndex);
    MAKOWRAPPER_API void fnCloseElement(MakoHandle elementHandle);

    // Layer APIs
    MAKOWRAPPER_API const wchar_t* fnGetLayerName(MakoHandle layerHandle);
    MAKOWRAPPER_API int fnGetLayerCount(MakoHandle documentHandle);
    MAKOWRAPPER_API MakoHandle fnGetLayer(MakoHandle documentHandle, int layerIndex);
    MAKOWRAPPER_API void fnFreeLayer(MakoHandle layerHandle);
    MAKOWRAPPER_API bool fnGetLayerIsVisible(MakoHandle layerHandle);
    MAKOWRAPPER_API void fnSetLayerIsVisible(MakoHandle layerHandle, bool isVisible);
    MAKOWRAPPER_API bool fnGetLayerIsLocked(MakoHandle layerHandle);
    MAKOWRAPPER_API void fnSetLayerIsLocked(MakoHandle layerHandle, bool isLocked);
    MAKOWRAPPER_API bool fnGetLayerIsPrintVisible(MakoHandle layerHandle);
    MAKOWRAPPER_API void fnAddLayerWithName(MakoHandle documentHandle, const wchar_t* layerName);

    // Separation APIs
    MAKOWRAPPER_API void fnLoadSeparations(MakoHandle documentHandle);
    MAKOWRAPPER_API int fnGetSeparationCount(MakoHandle documentHandle);
    MAKOWRAPPER_API bool fnGetSeparation(MakoHandle documentHandle, int index, MakoSeparation* makoSeparation);
    MAKOWRAPPER_API bool fnAddSeparation(MakoHandle documentHandle, MakoSeparation* makoSeparation);
    MAKOWRAPPER_API bool fnReplaceSeparation(MakoHandle documentHandle, int index, MakoSeparation* makoSeparation);
}

#endif //__MAKOWRAPPERDLL_H__
