﻿// -----------------------------------------------------------------------
//  <copyright file="ElementDeletionModifier.h" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#pragma once
#include "IDocumentModifier.h"

class ElementDeletionModifier : public IDocumentModifier
{
public:
    static IDocumentModifierPtr create(IPagePtr parentPage, int pageIndex, const CEDLVector<IDOMNodePtr>& nodesToRemove);

    bool undo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker) override;
    bool redo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker) override;

private:
    ElementDeletionModifier(IPagePtr parentPage, int pageIndex, const CEDLVector<IDOMNodePtr>& nodesToRemove);
    static bool haveAllMoved(const CEDLVector<bool>& haveMoved);

    class NodeData
    {
    public:
        NodeData()
        {
        }

        NodeData(const IDOMNodePtr& node) : m_node(node)
        {
            m_parent = node->getParentNode();
            m_previousSibling = node->getPreviousSibling();

            if (!m_parent)
                throw std::invalid_argument("The parameter 'node' must have a parent.");
        }

        bool isSiblingUnparented() const
        {
            return m_previousSibling && !m_previousSibling->getParentNode();
        }

        bool reinsertNode() const
        {
            return m_parent->insertChild(m_previousSibling, m_node);
        }

        void extract() const
        {
            m_parent->extractChild(m_node);
        }

        IDOMNodePtr getNode() const
        {
            return m_node;
        }

    private:
        IDOMNodePtr m_node;
        IDOMNodePtr m_parent;
        IDOMNodePtr m_previousSibling;
    };

    int m_pageIndex;
    IPagePtr m_parentPage;
    CEDLVector<NodeData> m_nodeData;
};
