﻿// -----------------------------------------------------------------------
//  <copyright file="ElementInsertionModifier.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "ElementInsertionModifier.h"
#include "MakoElement.h"
#include <edl/idompath.h>

ElementInsertionModifier::ElementInsertionModifier(
    const IPagePtr parentPage,
    const int pageIndex,
    const IDOMNodePtr newNode,
    const IDOMNodePtr parentNode,
    MakoLayerPtr targetLayer) : m_parentPage(parentPage), m_pageIndex(pageIndex), m_newNode(newNode), m_parentNode(parentNode), m_targetLayer(targetLayer)
{
    if (!parentPage)
        throw std::invalid_argument("The parameter 'parentPage' cannot be null.");

    if (!newNode)
        throw std::invalid_argument("The parameter 'newNode' cannot be null.");

    if (!parentNode)
        throw std::invalid_argument("The parameter 'parentNode' cannot be null.");

    if (pageIndex < 0)
        throw std::invalid_argument("The parameter 'pageIndex' must be equal or greater than 0.");

    // Mark the parent page as changed so that it's kept in memory and save picks up the changes.
    m_parentPage->edit();
}

IDocumentModifierPtr ElementInsertionModifier::create(const IPagePtr page, const int pageIndex, const IDOMNodePtr newNode, const IDOMNodePtr parentNode, MakoLayerPtr targetLayer)
{
    IDocumentModifier* documentModifier = new ElementInsertionModifier(page, pageIndex, newNode, parentNode, targetLayer);
    return IDocumentModifierPtr(documentModifier);
}

bool ElementInsertionModifier::undo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    m_parentNode->extractChild(m_newNode);
    changeTracker->registerChangeType(eSCPages);
    changeTracker->registerChangedPage(m_pageIndex);

    return true;
}

bool ElementInsertionModifier::redo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    m_parentNode->appendChild(m_newNode);
    m_newNode = m_targetLayer ? m_targetLayer->addReferenceToNode(m_newNode) : m_newNode;

    changeTracker->registerChangeType(eSCPages);
    changeTracker->registerChangedElement(new MakoElement(m_parentPage, m_pageIndex, m_newNode, stateTracker));

    return true;
}
