﻿// -----------------------------------------------------------------------
//  <copyright file="ElementTransformModifier.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "ElementTransformModifier.h"
#include <jawsmako/jawsmako.h>
#include <edl/idompath.h>
#include <edl/idomglyphs.h>
#include "MakoElement.h"

using namespace JawsMako;

ElementTransformModifier::ElementTransformModifier(const IPagePtr parentPage, const int pageIndex, const IDOMNodePtr node, const FMatrix originalNodeTransform, const FMatrix newNodeTransform) : m_parentPage(parentPage), m_node(node), m_originalNodeTransform(originalNodeTransform), m_newNodeTransform(newNodeTransform), m_pageIndex(pageIndex)
{
    if (!parentPage)
        throw std::invalid_argument("The parameter 'parentPage' cannot be null.");

    if (!node)
        throw std::invalid_argument("The parameter 'node' cannot be null.");

    if (pageIndex < 0)
        throw std::invalid_argument("The parameter 'pageIndex' must be equal or greater than 0.");

    // Mark the parent page as changed so that it's kept in memory and save picks up the changes.
    m_parentPage->edit();
}

IDocumentModifierPtr ElementTransformModifier::create(const IPagePtr parentPage, const int pageIndex, const IDOMNodePtr node, const FMatrix originalNodeTransform, const FMatrix newNodeTransform)
{
    IDocumentModifier* elementmodifier = new ElementTransformModifier(parentPage, pageIndex, node, originalNodeTransform, newNodeTransform);
    return IDocumentModifierPtr(elementmodifier);
}

bool ElementTransformModifier::undo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    // Set the translated render transform
    setNodeRenderTransform(m_node, m_originalNodeTransform);

    changeTracker->registerChangeType(eSCPages);
    changeTracker->registerChangedElement(new MakoElement(m_parentPage, m_pageIndex, m_node, stateTracker));

    return true;
}

bool ElementTransformModifier::redo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    // Set the translated render transform
    setNodeRenderTransform(m_node, m_newNodeTransform);

    changeTracker->registerChangeType(eSCPages);
    changeTracker->registerChangedElement(new MakoElement(m_parentPage, m_pageIndex, m_node, stateTracker));

    return true;
}


void ElementTransformModifier::setNodeRenderTransform(const IDOMNodePtr& node, const FMatrix matrix)
{
    IDOMPathNodePtr pathNode = edlobj2IDOMPathNode(node);
    if (pathNode)
    {
        pathNode->setRenderTransform(matrix);
        return;
    }

    IDOMGlyphsPtr glyphsNode = edlobj2IDOMGlyphs(node);
    if (glyphsNode)
    {
        glyphsNode->setRenderTransform(matrix);
    }
}
