﻿// -----------------------------------------------------------------------
//  <copyright file="HitDetection.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "HitDetection.h"
#include <jawsmako/transforms.h>
#include "HitDetectionResult.h"
#include "MakoDocument.h"

using namespace JawsMako;

bool HitDetection::findNodeInternal(IDOMNodePtr node, CTransformState state, const FPoint pagePoint, IDOMNodePtr& selectedNode, bool& selectedNodeFound, HitDetectionResult& result)
{
    // Enter the node
    CTransformState currentState = state.stateInsideNode(node);

    // Have we any children?
    IDOMNodePtr child = node->getLastChild();
    if (child)
    {
        // Check the kids - reverse order so the top is selected first
        while (child)
        {
            if (tryFindNode(child, currentState, pagePoint, selectedNode, selectedNodeFound, result))
                return true;

            child = child->getPreviousSibling();
        }
    }
    else
    {
        // Leaf nodes.

        // If we're a glyphs node, let's check for an actual hit.
        if (isNodeSelectable(node->getNodeType()))
        {
            FRect bounds = node->getBounds(false);

            // Get the bounds in terms of the page coordinates
            currentState.transform.transformRect(bounds);

            if (!bounds.containsPoint(pagePoint.x, pagePoint.y))
                return false;

            if (selectedNode && !selectedNodeFound)
            {
                if (selectedNode == node)
                    selectedNodeFound = true;

                return false;
            }

            HitDetectionResult hitResult;
            hitResult.node = node;
            hitResult.transform = currentState.transform;

            result = hitResult;
            return true;
        }
    }

    return false;
}

bool HitDetection::tryFindNode(const IDOMNodePtr node, const CTransformState state, const FPoint pagePoint, IDOMNodePtr& selectedNode, bool& selectedNodeFound, HitDetectionResult& result)
{
    try
    {
        return findNodeInternal(node, state, pagePoint, selectedNode, selectedNodeFound, result);
    }
    catch (...)
    {
        return false;
    }
}

bool HitDetection::findNode(const IDOMNodePtr node, const CTransformState state, const FPoint pagePoint, HitDetectionResult& result)
{
    IDOMNodePtr nullNode;
    return findNode(node, state, pagePoint, nullNode, result);
}

bool HitDetection::findNode(const IDOMNodePtr node, const CTransformState state, const FPoint pagePoint, IDOMNodePtr& selectedNode, HitDetectionResult& result)
{
    bool selectedNodeFound = false;
    const bool foundNode = tryFindNode(node, state, pagePoint, selectedNode, selectedNodeFound, result);
    
    // If we have previously selected nodes, and we haven't found a node.
    if (!foundNode && selectedNode)
    {
        // Try looking again without the previously selected node.
        return findNode(node, state, pagePoint, result);
    }

    return foundNode;
}
