﻿// -----------------------------------------------------------------------
//  <copyright file="IContourGenerator.h" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#pragma once

#include "jawsmako/jawsmako.h"
#include <edl/idompath.h>
#include "clipper.hpp"
#include "IOffsetSource.h"
#include "Types.h"

using namespace JawsMako;

class IContourGenerator;
typedef std::shared_ptr<IContourGenerator> IContourGeneratorPtr;

class IContourGenerator
{
public:
    virtual ~IContourGenerator();
    explicit IContourGenerator(const IJawsMakoPtr& mako, const MakoContourSettings& settings, const MakoSeparation& separation);

    static IContourGeneratorPtr create(const IJawsMakoPtr& mako, const MakoContourSettings& settings, const MakoSeparation& separation);
    IDOMNodePtr createContour(const std::vector<IDOMNodePtr>& nodes) const;
    IDOMNodePtr createPresetShapeContour(const IPagePtr& page, eShapeType type, MakoPoint* const makoPoints, int length) const;

private:
    IDOMPathNodePtr createContourPath(IDOMPathGeometryPtr geometry, IDOMBrushPtr brush, bool isPerfCut) const;
    IDOMPathNodePtr createPath() const;
    void addContour(IDOMPathGeometryPtr geometry, ClipperLib::Path path) const;
    void processChildren(IDOMPathGeometryPtr geometry, ClipperLib::PolyNodes nodes) const;
    void fillPath(IDOMPathNodePtr& pathNode, const ClipperLib::PolyTree& offsetTree) const;
    void addNodeToOffsetter(IOffsetSourcePtr& offsetSource, ClipperLib::ClipperOffset& offsetter) const;
    static FPoint intPointToFPoint(ClipperLib::IntPoint point);
    ClipperLib::JoinType getJointType() const;
    void addEllipseGeometry(const IDOMPathGeometryPtr& geometry, const CEDLVector<FPoint>& points) const;
    void addPolygonGeometry(const IDOMPathGeometryPtr& geometry, const CEDLVector<FPoint>& points) const;
    static ClipperLib::Path::value_type fPointToIntPoint(FPoint point);
    IDOMArcSegmentPtr createEllipseSegment(const FPoint& endPoint, double radiusX, double radiusY) const;

    const IJawsMakoPtr m_mako;
    MakoContourSettings m_settings;
    MakoSeparation m_separation;
};

// Convert color value from 0.0 to 1.0 to 0 to 255
#define F2B(f) ((f) >= 1.0 ? 255 : (int)((f)*256.0))

// Convert color value from 0 to 255 to 0.0 to 1.0
#define B2F(b) ((float)(b) / 256.0f)
