﻿// -----------------------------------------------------------------------
//  <copyright file="IOffsetSource.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "IOffsetSource.h"
#include <jawsmako/jawsmako.h>
#include <edl/idomglyphs.h>
#include "GlyphsOffsetNode.h"
#include "PathOffsetNode.h"
#include <jawsmako/transforms.h>

using namespace JawsMako;

IOffsetSourcePtr IOffsetSource::create(IDOMNodePtr node)
{
    switch (node->getNodeType())
    {
    case eDOMGlyphsNode:
        return GlyphsOffsetNode::create(edlobj2IDOMGlyphs(node));
    case eDOMPathNode:
        return PathOffsetNode::create(edlobj2IDOMPathNode(node));
    default:
        throw std::invalid_argument("Unexpected node type. Cannot create contour generator.");
    }
}

IDOMPathGeometryPtr IOffsetSource::createFlattenedGeometry(IJawsMakoPtr mako, const eContourAlignment alignment)
{
    const IDOMPathNodePtr pathNode = getPathData();

    IDOMPathGeometryPtr pathGeometry;
    switch (alignment)
    {
    case eOutline:
        if (!pathNode->getPathData(pathGeometry))
            throw std::runtime_error("Couldn't get path geometry from path node.");
        break;

    case eBoundingBox:
        pathGeometry = IDOMPathGeometry::create(mako, pathNode->getBounds(false));
        break;

    default:
        throw std::runtime_error("Unknown eContourAlignment.");
    }

    // Flatten using the transform state to transform the geometry back into page coordinates
    const CTransformState transformState = CTransformState(pathNode);
    return pathGeometry->getFlattenedGeometry(transformState.transform, 600, mako);
}
