﻿// -----------------------------------------------------------------------
//  <copyright file="LayerAddModifier.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "LayerAddModifier.h"


LayerAddModifier::LayerAddModifier(IJawsMakoPtr jawsMako, MakoDocument* const document, const String& layerName) :
    m_mako(jawsMako), m_document(document), m_layerName(layerName)
{
    if (jawsMako == nullptr)
        throw std::invalid_argument("Parameter 'jawsMako' cannot be null");

    if (document == nullptr)
        throw std::invalid_argument("Parameter 'document' cannot be null");
}

IDocumentModifierPtr LayerAddModifier::create(IJawsMakoPtr jawsMako, MakoDocument* const document, const String& layerName)
{
    IDocumentModifier* modifier = new LayerAddModifier(jawsMako, document, layerName);
    return IDocumentModifierPtr(modifier);
}

bool LayerAddModifier::undo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    m_document->removeLayerWithName(m_layerName);
    changeTracker->registerChangeType(eSCLayers);
    return true;
}

bool LayerAddModifier::redo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    if (!m_layer)
    {
        m_document->addLayerWithName(m_layerName);
        m_layer = MakoLayerPtr(m_document->getLayerWithName(m_layerName));
    }
    else
    {
        m_document->addLayer(m_layer);
    }

    changeTracker->registerChangeType(eSCLayers);
    return true;
}
