﻿// -----------------------------------------------------------------------
//  <copyright file="LayerLockStateModifier.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include <vector>
#include "LayerLockStateModifier.h"

LayerLockStateModifier::LayerLockStateModifier(IJawsMakoPtr mako, IOptionalContentPtr optionalContent, IOptionalContentGroupPtr optionalContentGroup, int lockGroupIndex, bool newLayerLockState) : 
m_optionalContent(optionalContent), m_optionalContentGroup(optionalContentGroup), m_mako(mako), m_groupIndex(lockGroupIndex), m_newLayerLockState(newLayerLockState)
{
    if (mako == nullptr)
        throw std::invalid_argument("Parameter 'mako' cannot be null");

    if (optionalContent == nullptr)
        throw std::invalid_argument("Parameter 'optionalContent' cannot be null");

    if (optionalContentGroup == nullptr)
        throw std::invalid_argument("Parameter 'optionalContentGroup' cannot be null");

    if (m_groupIndex == -1 && !newLayerLockState)
        throw std::invalid_argument("Invalid 'm_groupIndex' for unlocking.");
}

IDocumentModifierPtr LayerLockStateModifier::create(IJawsMakoPtr mako, IOptionalContentPtr optionalContent, IOptionalContentGroupPtr optionalContentGroup, int lockGroupIndex, bool newLayerLockState)
{
    IDocumentModifier* layerModifier = new LayerLockStateModifier(mako, optionalContent, optionalContentGroup, lockGroupIndex, newLayerLockState);
    return IDocumentModifierPtr(layerModifier);
}

void LayerLockStateModifier::updateLocktate(const bool lockState) 
{
    const IOptionalContentGroupReferencePtr optionalContentGroupReference = m_optionalContentGroup->getReference();
    const IOptionalContentConfigurationPtr optionalContentConfiguration = m_optionalContent->getDefaultConfiguration();
    COptionalContentGroupReferenceVect lockedGroups = optionalContentConfiguration->getLockedGroups();

    // If group is not in the locked group list, we will insert it to the last.
    if (m_groupIndex == -1)
        m_groupIndex = lockedGroups.size();

    if (lockState)
        lockedGroups.insert(m_groupIndex, optionalContentGroupReference);
    else
        lockedGroups.erase(m_groupIndex);

    optionalContentConfiguration->setLockedGroups(lockedGroups);
}

bool LayerLockStateModifier::undo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    updateLocktate(!m_newLayerLockState);
    changeTracker->registerChangeType(eSCLayers);

    return true;
}

bool LayerLockStateModifier::redo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    updateLocktate(m_newLayerLockState);
    changeTracker->registerChangeType(eSCLayers);

    return true;
}
