﻿// -----------------------------------------------------------------------
//  <copyright file="LayerVisibilityModifier.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "LayerVisibilityModifier.h"
#include <vector>

LayerVisibilityModifier::LayerVisibilityModifier(IJawsMakoPtr mako, IOptionalContentPtr optionalContent, IOptionalContentGroupPtr optionalContentGroup, bool newLayerVisibility) : 
m_optionalContent(optionalContent), m_newLayerVisibility(newLayerVisibility), m_optionalContentGroup(optionalContentGroup), m_mako(mako)
{
    if (mako == nullptr)
        throw std::invalid_argument("Parameter 'mako' cannot be null");

    if (optionalContent == nullptr)
        throw std::invalid_argument("Parameter 'optionalContent' cannot be null");

    if (optionalContentGroup == nullptr)
        throw std::invalid_argument("Parameter 'optionalContentGroup' cannot be null");
}

IDocumentModifierPtr LayerVisibilityModifier::create(IJawsMakoPtr mako, IOptionalContentPtr optionalContent, IOptionalContentGroupPtr optionalContentGroup, bool newLayerVisibility)
{
    IDocumentModifier* layerModifier = new LayerVisibilityModifier(mako, optionalContent, optionalContentGroup, newLayerVisibility);
    return IDocumentModifierPtr(layerModifier);
}

void LayerVisibilityModifier::updateState(const bool state) const
{
    setGroupVisible(state);
}

bool LayerVisibilityModifier::undo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    updateState(!m_newLayerVisibility);
    changeTracker->registerChangeType(eSCLayers);

    return true;
}

bool LayerVisibilityModifier::redo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    updateState(m_newLayerVisibility);
    changeTracker->registerChangeType(eSCLayers);

    return true;
}

// This is adapted from Mako's forceGroupState() without resetting the group's intent and usage.
void LayerVisibilityModifier::setGroupVisible(bool visible) const
{
    const IOptionalContentGroupReferencePtr groupRef = m_optionalContentGroup->getReference();

    // We only support the default configuration for querying locked status. 
    // Even Acrobat doesn't currently have a method for setting multiple configurations.
    const IOptionalContentConfigurationPtr defaultConfiguration = m_optionalContent->getDefaultConfiguration();

    // Add / remove from the configuration's visible/invisible lists
    if (visible)
    {
        if (!defaultConfiguration->groupInVisibleGroups(groupRef))
        {
            COptionalContentGroupReferenceVect groups = defaultConfiguration->getDefaultVisibleGroups();
            groups.append(groupRef);
            defaultConfiguration->setDefaultVisibleGroups(groups);
        }
        if (defaultConfiguration->groupInInvisibleGroups(groupRef))
        {
            // Remove
            COptionalContentGroupReferenceVect groups = defaultConfiguration->getDefaultInvisibleGroups();
            for (uint32 i = 0; i < groups.size(); i++)
            {
                if (groups[i]->equals(groupRef))
                {
                    groups.erase(i);
                }
                defaultConfiguration->setDefaultInvisibleGroups(groups);
            }
        }
    }
    else
    {
        if (!defaultConfiguration->groupInInvisibleGroups(groupRef))
        {
            COptionalContentGroupReferenceVect groups = defaultConfiguration->getDefaultInvisibleGroups();
            groups.append(groupRef);
            defaultConfiguration->setDefaultInvisibleGroups(groups);
        }
        if (defaultConfiguration->groupInVisibleGroups(groupRef))
        {
            // Remove
            COptionalContentGroupReferenceVect groups = defaultConfiguration->getDefaultVisibleGroups();
            for (uint32 i = 0; i < groups.size(); i++)
            {
                if (groups[i]->equals(groupRef))
                {
                    groups.erase(i);
                }
                defaultConfiguration->setDefaultVisibleGroups(groups);
            }
        }
    }
}
