﻿// -----------------------------------------------------------------------
//  <copyright file="MakoColorReplacementTransformImplementation.h" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------
#ifndef MAKOCOLORREPLACEMENTTRANSFORMIMPLEMENTATION_H
#define MAKOCOLORREPLACEMENTTRANSFORMIMPLEMENTATION_H

#include "customtransform.h"
#include <map>

using namespace JawsMako;
using namespace EDL;

typedef std::map<IDOMColorSpacePtr, IDOMColorSpacePtr> MakoColorSpaceMap;

class MakoColorReplacementTransformImplementation : public ICustomTransform::IImplementation
{
public:
    MakoColorReplacementTransformImplementation(const IJawsMakoPtr &jawsMako, const IRendererTransform::CInkInfoVect &allInks, const IRendererTransform::CInkInfo &oldInk, const IRendererTransform::CInkInfo &newInk);
    ~MakoColorReplacementTransformImplementation();
    IDOMBrushPtr transformBrush(IImplementation *genericImplementation, const IDOMBrushPtr &brush, eBrushUsage usage, const CTransformState &state) override;
    IDOMColorSpacePtr transformColorSpace(IImplementation *, const IDOMColorSpacePtr &colorSpace, const CTransformState &state) override;

private:
    static bool colorSpaceHasComponent(const IDOMColorSpaceDeviceNPtr& colorSpace, const U8String& colorantToFind);
    void getColorantInfoFromInkInfo(IDOMColorSpaceDeviceN::CColorantInfo* pColorantInfo, const IRendererTransform::CInkInfo & inkInfo, const IDOMColorSpacePtr& colorSpace) const;
    void getSpecialNoneColorantInfo(IDOMColorSpaceDeviceN::CColorantInfo* pColorantInfo, const IDOMColorSpacePtr& colorSpace) const;
    void getSpecialAllColorantInfo(IDOMColorSpaceDeviceN::CColorantInfo* pColorantInfo, const IDOMColorSpacePtr& colorSpace) const;
    IDOMColorSpacePtr replaceColorSpace(IDOMColorSpacePtr colorSpace);
    IDOMColorSpacePtr replaceDeviceNColorSpace(const IDOMColorSpaceDeviceNPtr& deviceN);
    IDOMColorSpacePtr replaceIndexedColorSpace(const IDOMColorSpaceIndexedPtr& indexedColorSpace);

    IRendererTransform::CInkInfoVect m_inkList;
    IJawsMakoPtr m_mako;
    U8String m_colorant;
    MakoColorSpaceMap m_colorSpaceMap;
    IRendererTransform::CInkInfo m_newInk;
};

#endif

