﻿// -----------------------------------------------------------------------
//  <copyright file="MakoDebug.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
//  <summary>
//  This example is provided on an "as is" basis and without warranty of any kind.
//  Global Graphics Software Ltd. does not warrant or make any representations regarding the use or
//  results of use of this example.
//  </summary>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "MakoDebug.h"

#include "GlyphsOffsetNode.h"

#include <iostream>
#include <vector>

using namespace JawsMako;
using namespace EDL;


static IDOMPathGeometryPtr getNodeClip(const IDOMNodePtr& node)
{
    IDOMPathGeometryPtr clip;

    const IDOMPathNodePtr pathNode = edlobj2IDOMPathNode(node);
    if (pathNode)
    {
        (bool)pathNode->getClip(clip);
    }
    else
    {
        const IDOMGlyphsPtr glyphsNode = edlobj2IDOMGlyphs(node);
        if (glyphsNode)
            (bool)glyphsNode->getClip(clip);
    }

    return clip;
}

std::string getNodeType(const IDOMNodePtr& node)
{
    switch (node->getNodeType())
    {
    case eDOMGroupNode: return "eDOMGroupNode";
    case eDOMPathNode: return "eDOMPathNode";
    case eDOMTransparencyGroupNode: return "eDOMTransparencyGroupNode";
    case eDOMFixedPageNode: return "eDOMFixedPageNode";
    case eDOMFormInstanceNode: return "eDOMFormInstanceNode";
    case eDOMGlyphNode: return "eDOMGlyphNode";
    case eDOMGlyphsNode: return "eDOMGlyphsNode";
    default:
        return "Other";
    }
}

void dumpNodeInfo(const IDOMNodePtr& node, bool markNode, int depth)
{
    for (int i = 0; i < depth; i++)
        std::cout << "    ";
    if (markNode)
        std::cout << "->";

    std::cout << getNodeType(node);

    if (getNodeClip(node))
        std::cout << " (Clipped)";

    std::cout << std::endl;
}

void dumpNodeTree(const IDOMNodePtr& node, int markAtDepth, int depth)
{
    dumpNodeInfo(node, markAtDepth == depth, depth);
    if (!node->hasChildNodes())
        return;

    IDOMNodePtr firstChild = node->getFirstChild();
    dumpNodeTree(firstChild, -1, depth + 1);

    IDOMNodePtr idomNode = firstChild;
    while ((idomNode = node->getNextChild(idomNode)) != nullptr && idomNode != firstChild)
        dumpNodeTree(idomNode, -1, depth + 1);
}

void dumpNodeInTree(const IDOMNodePtr& node)
{
    if (!node)
        return;

    std::vector<IDOMNodePtr> parents;
    IDOMNodePtr n = node;
    while (n->getParentNode())
    {
        n = n->getParentNode();
        parents.push_back(n);
    }

    int depth = 0;
    for (int i = (int)parents.size() - 1; i >= 0; i--)
        dumpNodeInfo(parents[i], false, depth++);

    dumpNodeTree(node, depth, depth);
}
