// -----------------------------------------------------------------------
//  <copyright file="MakoDocument.h" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------


#ifndef MAKODOCUMENT_H
#define MAKODOCUMENT_H

#include "MakoElement.h"
#include "MakoLayer.h"
#include <jawsmako/transforms.h>
#include "StateTracker.h"
#include "IPageTransformer.h"

using namespace JawsMako;

class MakoDocument
{
public:
    MakoDocument(IJawsMakoPtr mako, IDocumentPtr document, std::wstring sourceFilePath);

    IPagePtr getPage(int pageIndex) const;
    double getPageHeight(int pageIndex) const;
    double getPageWidth(int pageIndex) const;
    uint32 getPageCount() const;
    uint32 getLayerCount() const;
    uint32 getPageIndex(const IPagePtr& page) const;  
    MakoElement* getPageElement(int pageIndex, double x, double y, IDOMNodePtr selectedNode) const;
    CEDLVector<MakoElement*> getAllPageElements(int pageIndex) const;
    MakoLayer* getLayerAtIndex(int layerIndex) const;
    MakoLayer* getLayerWithName(const String& layerName) const;
    void addLayer(const MakoLayerPtr& layer) const;
    void addLayerWithName(const String& layerName) const;
    void removeLayerWithName(const String& layerName) const;
    IOptionalContentPtr getOptionalContent() const;
    StateTrackerPtr getStateTracker() const;
    void addElement(const MakoElement* element) const;
    
private:
    StateTrackerPtr m_stateTracker;
    IDocumentPtr m_document;
    IJawsMakoPtr m_mako;
    std::wstring m_sourceFilePath;
};

bool isNodeSelectable(eDOMNodeType nodeType);

#endif // MAKODOCUMENT_H
