// -----------------------------------------------------------------------
//  <copyright file="MakoElement.h" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#pragma once

#include "Types.h"
#include "StateTracker.h"
#include "MakoLayer.h"

using namespace JawsMako;

class MakoElement
{
public:
    MakoElement(IPagePtr page, int pageIndex, IDOMNodePtr node, StateTrackerPtr stateTracker);
    ~MakoElement();
    void getBounds(MakoRect* rect) const;
    void setBounds(const MakoRect& rect, bool isInvertedHorizontally = false, bool isInvertedVertically = false) const;
    IDOMNodePtr getNode() const;

    /**
     * If a cloned element had parents which are groups, this returns the root group node (Otherwise null).
     */
    IDOMNodePtr getGroupNode() const;

    int getPageIndex() const;
    MakoElement* copy(const IJawsMakoPtr& mako, int newPageIndex) const;
    
    static MakoElement* createContour(const IJawsMakoPtr& mako, std::vector<MakoElement*>& elements, const MakoContourSettings& settings, const MakoSeparation& separation, MakoLayerPtr targetLayer);
    static MakoElement* createPresetShapeContour(const IJawsMakoPtr& mako, const IPagePtr& page, int pageIndex, const StateTrackerPtr& stateTracker, eShapeType type, MakoPoint* points, int length, const MakoContourSettings& settings, const MakoSeparation& separation, MakoLayerPtr targetLayer);
    static void extractFromParent(const CEDLVector<MakoElement*>& elements);

private:
    static FMatrix getNodeRenderTransform(IDOMNodePtr node);
    static void setNodeRenderTransform(IDOMNodePtr node, const FMatrix& m);
    static void transformToPageCoordinates(FPoint& translation, IPagePtr page);
    static void transformToNodeCoordinates(FPoint& translation, IDOMNodePtr node);
    static FRect makoRectToRect(MakoRect makoRect);

    IDOMNodePtr m_groupNode;
    IDOMNodePtr m_node;
    IPagePtr m_page;
    StateTrackerPtr m_stateTracker;
    const int m_pageIndex;
};
