// -----------------------------------------------------------------------
//  <copyright file="MakoLayer.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "MakoLayer.h"
#include <jawsmako/optionalcontent.h>
#include "LayerVisibilityModifier.h"
#include "LayerLockStateModifier.h"

MakoLayer::MakoLayer(IJawsMakoPtr mako, const IOptionalContentPtr optionalContent, const IOptionalContentGroupPtr optionalContentGroup, StateTrackerPtr stateTracker) :
m_optionalContent(optionalContent), m_optionalContentGroup(optionalContentGroup), m_stateTracker(stateTracker), m_mako(mako)
{
}

MakoLayer::~MakoLayer()
{
}

String MakoLayer::getName() const
{
    const U8String name = m_optionalContentGroup->getName();
    return U8StringToString(name);
}

bool MakoLayer::isPrintVisible() const
{
    return m_optionalContent->groupIsVisible(m_optionalContentGroup, eOCEPrint);
}

bool MakoLayer::getIsLocked() const
{
    return getLockedGroupIndex() != -1;
}

int MakoLayer::getLockedGroupIndex() const
{
    const IOptionalContentGroupReferencePtr optionalContentGroupReference = m_optionalContentGroup->getReference();

    // We only support the default configuration for querying locked status. 
    // Even Acrobat doesn't currently have a method for setting multiple configurations.
    const IOptionalContentConfigurationPtr optionalContentConfiguration = m_optionalContent->getDefaultConfiguration();

    COptionalContentGroupReferenceVect lockedGroups = optionalContentConfiguration->getLockedGroups();
    for (uint32 groupIndex = 0; groupIndex < lockedGroups.size(); groupIndex++)
    {
        if (lockedGroups[groupIndex]->equals(optionalContentGroupReference))
            return groupIndex;
    }

    return -1;
}

void MakoLayer::setIsLocked(const bool isLocked) const
{
    // If already in the state, do nothing.
    if (getIsLocked() == isLocked)
        return;
    
    const IDocumentModifierPtr modifier = LayerLockStateModifier::create(m_mako, m_optionalContent, m_optionalContentGroup, getLockedGroupIndex(), isLocked);
    m_stateTracker->registerChange(modifier);
}

bool MakoLayer::getIsVisible() const
{
    return m_optionalContent->groupIsVisible(m_optionalContentGroup, eOCEView);
}

void MakoLayer::setIsVisible(const bool isVisible) const
{
    const IDocumentModifierPtr modifier = LayerVisibilityModifier::create(m_mako, m_optionalContent, m_optionalContentGroup, isVisible);
    m_stateTracker->registerChange(modifier);
}

IOptionalContentPtr MakoLayer::getOptionalContent() const
{
    return m_optionalContent;
}

IOptionalContentGroupPtr MakoLayer::getOptionalContentGroup() const
{
    return m_optionalContentGroup;
}

IDOMNodePtr MakoLayer::addReferenceToNode(const IDOMNodePtr& node) const
{
    return m_optionalContent->makeNodeOptional(node, m_optionalContentGroup);
}
