﻿// -----------------------------------------------------------------------
//  <copyright file="PageRotationTransformer.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "PageRotationTransformer.h"

PageRotationTransformer::PageRotationTransformer(JawsMako::IPagePtr page): m_page(page) {
}

JawsMako::FRect PageRotationTransformer::getTransformedPageRect()
{
    JawsMako::FRect pageRect = getPageRect();
    JawsMako::int32 rotation = m_page->getRotate();

    if (rotation == 90 || rotation == 270)
        return JawsMako::FRect(pageRect.y, pageRect.x, pageRect.dY, pageRect.dX);

    return pageRect;
}

JawsMako::FMatrix PageRotationTransformer::getPageTransform()
{
    JawsMako::FMatrix transform;

    const int rotationDegrees = (m_page->getRotate() + 360) % 360;

    JawsMako::FRect pageRect = getPageRect();

    const double width = pageRect.dX;
    const double height = pageRect.dY;
    switch (rotationDegrees / 90)
    {
    case 0: // no rotation.
        transform.setDX(-pageRect.x);
        transform.setDY(-pageRect.y);
        break;

    case 1: // 90 degrees
        transform.setDX(height + pageRect.y);
        transform.setDY(-pageRect.x);
        break;

    case 2: // 180 degrees
        transform.setDX(width + pageRect.x);
        transform.setDY(height + pageRect.y);
        break;

    case 3: // 270 degrees
        transform.setDX(-pageRect.y);
        transform.setDY(width + pageRect.x);
        break;

    default:
        break;
    }

    transform.rotate(rotationDegrees * 3.141592 / 180.0);

    return transform;
}