﻿// -----------------------------------------------------------------------
//  <copyright file="StateTracker.h" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#pragma once

#include <vector>
#include "IDocumentModifier.h"
#include "TransactionalModifier.h"

class StateTracker : public std::enable_shared_from_this<StateTracker>
{
public:

    static StateTrackerPtr create();
    virtual ~StateTracker();

    bool canUndo() const;
    bool canRedo() const;
    bool undo(ChangeTrackerPtr changeTracker);
    bool redo(ChangeTrackerPtr changeTracker);
    void registerChange(IDocumentModifierPtr documentModifier);
    void startChangeGroup();
    void completeChangeGroup();

private:
    
    // Private constructor as this always has to be created using the ::create method,
    // due to how it shares it's self as a shared_ptr instance.
    StateTracker();

    void resize(size_t position);
    std::vector<IDocumentModifierPtr> m_modifierStack;
    size_t m_position;
    TransactionalModifier* m_transactionalModifier;
};
