﻿// -----------------------------------------------------------------------
//  <copyright file="TransactionalModifier.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "TransactionalModifier.h"

void TransactionalModifier::registerChange(IDocumentModifierPtr modifier)
{
    m_modifierStack.push_back(modifier);
}

bool TransactionalModifier::undo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    bool rollingBack = false;
    for (size_t modifierIndex = m_modifierStack.size() - 1; modifierIndex < m_modifierStack.size() && modifierIndex >= 0; )
    {
        IDocumentModifierPtr modifier = m_modifierStack[modifierIndex];

        if (rollingBack)
        {
            modifier->redo(stateTracker, changeTracker);
            modifierIndex++;
            continue;
        }

        rollingBack = !modifier->undo(stateTracker, changeTracker);
        modifierIndex = rollingBack ? modifierIndex + 1 : modifierIndex - 1;
    }

    return !rollingBack;
}

bool TransactionalModifier::redo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    bool rollingBack = false;
    for (size_t modifierIndex = 0; modifierIndex < m_modifierStack.size() && modifierIndex >= 0; )
    {
        IDocumentModifierPtr modifier = m_modifierStack[modifierIndex];

        if (rollingBack)
        {
            modifier->undo(stateTracker, changeTracker);
            modifierIndex--;
            continue;
        }

        rollingBack = !modifier->redo(stateTracker, changeTracker);
        modifierIndex = rollingBack ? modifierIndex - 1 : modifierIndex + 1;
    }

    return !rollingBack;
}
