﻿// -----------------------------------------------------------------------
//  <copyright file="TransactionalModifier.h" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#pragma once
#include "IDocumentModifier.h"
#include <memory>
#include <vector>
#include "ChangeTracker.h"

/**
 * A TransactionModifier allows multiple modifiers to work in one transaction.
 * This means that if any modification added to this TransactionalModifier fails, 
 * the modifications made up until that point are rolled back and the original
 * state should be restored.
 */
class TransactionalModifier : public IDocumentModifier
{
public:
    void registerChange(IDocumentModifierPtr modifier);
    bool undo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker) override;
    bool redo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker) override;

private:
    std::vector<IDocumentModifierPtr> m_modifierStack;
};
