﻿// -----------------------------------------------------------------------
//  <copyright file="Types.h" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#pragma once

#include <memory>

class MakoDocument;
typedef std::shared_ptr<MakoDocument> MakoDocumentPtr;

extern "C"
{
    typedef void* MakoHandle;
    typedef int MakoResult;

    typedef enum 
    {
        eEllipse,
        ePolygon
    } eShapeType;

    typedef enum 
    {
        eOutline,
        eBoundingBox
    } eContourAlignment;

    typedef struct MakoContourSettings
    {
        double offsetInPoints;
        bool removeHoles;
        const wchar_t* layerName;
        eContourAlignment alignment;

    } MakoContourSettings;

    typedef struct MakoRender
    {
        void* buffer;
        int bufferStride;
        int width;
        int height;
        int samplePerPixel;

    } MakoRaster;

    typedef struct MakoPoint
    {
        double x;
        double y;

    } MakoPoint;

    typedef struct MakoRect
    {
        double x;
        double y;
        double width;
        double height;

    } MakoRect;

    typedef struct MakoMatrix
    {
        double m11;
        double m12;
        double m21;
        double m22;
        double offsetX;
        double offsetY;

    } MakoMatrix;

    typedef struct MakoSize
    {
        double width;
        double height;

    } MakoSize;

    typedef struct MakoSeparation
    {
        const wchar_t * name;
        bool isProcessColor;
        int colorArgbValue;

    } MakoSeparation;
}
