﻿using System;
using System.IO;
using Mako.Wrapper;

namespace MakoConverter
{
    class Program
    {
        static int Main(string[] args)
        {
            // There should be at least two arguments; a source file and a destination file.
            if (args.Length < 2)
            {
                Console.WriteLine("Usage: <source file> <output file> [ parameter=value parameter=value ... ]");
                return 1;
            }

            var inputFilePath = args[0];
            var outputFilePath = args[1];

            // Gather parameters into a list
            var outputParameters = new string[args.Length - 2];
            var parameterCount = 0;
            for (var i = 2; i < args.Length; i++)
            {
                if (args[i].Contains("="))
                    outputParameters[parameterCount++] = args[i];
            }

            // Create a Mako instance
            MakoInstance makoInstance = MakoInstance.Instance;
            if (makoInstance == null)
                throw new InvalidOperationException("Mako is not initialized.");
            
            // Check input file is there
            if (!File.Exists(inputFilePath))
                throw new FileNotFoundException("Cannot find path to PDF.", inputFilePath);

            // Read source document into an assembly
            IMakoAssembly makoAssembly = makoInstance.Open(inputFilePath);

            // Get document (not needed for conversion)
            IMakoDocument makoDocument = makoAssembly.GetDocument();
            var pageCount = makoDocument.GetPageCount();
            var pageLabel = pageCount == 1 ? "page" : "pages"; 
            Console.WriteLine($"The document {inputFilePath} has {pageCount} {pageLabel}.");

            // Report page size. Mako's units are 1/96"
            var pageSize = makoDocument.GetPageSize(0);
            Console.WriteLine($"The first page is {pageSize.width / 96 * 25.4:0.00}mm wide by {pageSize.height / 96 * 25.4:0.00}mm high.");

            // Write assempby to output file
            // File extension determines file type
            makoAssembly.Save(outputFilePath, outputParameters);
            return 0;
        }
    }
}
